<?php

namespace common\logic\Distribution\Distribution;

use common\helpers\GlobalParam;
use common\logic\BaseService;
use common\logic\BuilderInterface;
use common\logic\Distribution\PointSaleDistribution\PointSaleDistribution;
use common\logic\Distribution\PointSaleDistribution\PointSaleDistributionBuilder;
use common\logic\Distribution\PointSaleDistribution\PointSaleDistributionRepository;
use common\logic\Distribution\ProductDistribution\ProductDistribution;
use common\logic\Order\Order\OrderModel;
use common\logic\Producer\Producer\Producer;
use common\logic\Product\Product\Product;

class DistributionBuilder extends BaseService implements BuilderInterface
{
    protected DistributionRepository $distributionRepository;
    protected PointSaleDistributionRepository $pointSaleDistributionRepository;
    protected PointSaleDistributionBuilder $pointSaleDistributionBuilder;

    public function __construct()
    {
        $this->distributionRepository = $this->loadService(DistributionRepository::class);
        $this->pointSaleDistributionRepository = $this->loadService(PointSaleDistributionRepository::class);
        $this->pointSaleDistributionBuilder = $this->loadService(PointSaleDistributionBuilder::class);
    }

    public function instanciate(Producer $producer, string $date, bool $delivery = true): Distribution
    {
        $distribution = new Distribution();

        $distribution->id_producer = $producer->id;
        $distribution->date = $date;
        $distribution->delivery = (int) $delivery;

        return $distribution;
    }

    public function create(Producer $producer, string $date, int $delivery = 1): Distribution
    {
        $distribution = $this->instanciate($producer, $date, 1);
        $distribution->save();

        return $distribution;
    }

    public function createIfNotExist(Producer $producer, string $date, int $delivery = 1)
    {
        $distribution = $this->distributionRepository->getOne($producer, $date);

        if (!$distribution) {
            $distribution = $this->create($producer, $date, 1);
        }

        return $distribution;
    }

    /**
     * Initialise un jour de production.
     */
    // initDistribution
    public function init(string $date, Producer $producer = null): Distribution
    {
        $distribution = null;
        if ($date != '') {
            if (!$producer) {
                $producer = GlobalParam::getCurrentProducer();
            }
            $distribution = $this->createIfNotExist($producer, $date, 1);
        }

        if ($distribution) {
            $this->initPointSaleDistribution($distribution);
            $this->initProductDistribution($distribution);
        }

        return $distribution;
    }

    public function initPointSaleDistribution(Distribution $distribution): void
    {
        $countPointSaleDistribution = $this->pointSaleDistributionRepository->countByDistribution($distribution);
        if (!$countPointSaleDistribution) {
            $this->pointSaleDistributionBuilder->initAll($distribution, true);
        }
    }

    public function initProductDistribution(Distribution $distribution)
    {
        $products = Product::searchAll();

        $productsDistribution = ProductDistribution::searchAll([
            'id_distribution' => $distribution->id
        ]);
        if (!count($productsDistribution)) {
            foreach ($products as $product) {
                $this->linkProduct($distribution, $product);
            }
        }
    }

    /**
     * Lie un produit à la distribution.
     */
    public function linkProduct(Distribution $distribution, Product $product): ProductDistribution
    {
        $productDistribution = ProductDistribution::searchOne([
            'id_distribution' => $this->id,
            'id_product' => $product->id
        ]);

        if (!$productDistribution) {
            $productDistribution = new ProductDistribution();
            $productDistribution->id_distribution = $this->id;
            $productDistribution->id_product = $product->id;
        }

        $dayDistribution = date('N', strtotime($this->date));
        $daysArray = [
            1 => 'monday',
            2 => 'tuesday',
            3 => 'wednesday',
            4 => 'thursday',
            5 => 'friday',
            6 => 'saturday',
            7 => 'sunday',
        ];

        $productDistribution->active = 0;
        $day = $daysArray[$dayDistribution];
        if ($product->active && $product->$day) {
            $productDistribution->active = 1;
        }

        $productDistribution->quantity_max = $product->quantity_max;
        $fieldQuantityMax = 'quantity_max_' . $day;
        if (isset($product->$fieldQuantityMax) && $product->$fieldQuantityMax > 0) {
            $productDistribution->quantity_max = $product->$fieldQuantityMax;
        }
        $productDistribution->save();

        // update prices product order
        $ordersArray = OrderModel::searchAll([
            'distribution.date' => $this->date,
            'distribution.id_producer' => $this->id_producer
        ],
            [
                'conditions' => 'date_delete IS NULL AND origin != "user"'
            ]);
        if ($ordersArray) {
            foreach ($ordersArray as $order) {
                foreach ($order->productOrder as $productOrder) {
                    if ($productOrder->id_product == $product->id) {
                        $productOrder->price = $product->price;

                        $user = false;
                        $userProducer = false;
                        if (isset($order->user) && $order->user) {
                            $user = $order->user;
                            $userProducer = UserProducerModel::searchOne([
                                'id_user' => $user->id,
                                'id_producer' => $order->distribution->id_producer
                            ]);
                        }

                        $productOrder->price = $product->getPrice([
                            'user' => $user,
                            'user_producer' => $userProducer,
                            'point_sale' => $order->pointSale,
                            'quantity' => $productOrder->quantity
                        ]);

                        $productOrder->save();
                    }
                }
            }
        }

        return $productDistribution;
    }
}